# Copyright (C) 2014-2015 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

import logging
import os.path

from vmcloak.abstract import OperatingSystem
from vmcloak.rand import random_string
from vmcloak.verify import valid_serial_key

log = logging.getLogger(__name__)


class Windows7(OperatingSystem):
    name = 'win7'
    service_pack = 2
    mount = '/mnt/win7'
    nictype = '82540EM'
    osdir = os.path.join('sources', '$oem$', '$1')
    genisoargs = [
        '-no-emul-boot', '-iso-level', '2', '-udf', '-J', '-l', '-D', '-N',
        '-joliet-long', '-relaxed-filenames',
    ]

    def _autounattend_xml(self):
        values = {
            'PRODUCTKEY': self.serial_key,
            'COMPUTERNAME': random_string(8, 16),
        }

        buf = open(os.path.join(self.path, 'autounattend.xml'), 'rb').read()
        for key, value in values.items():
            buf = buf.replace('@%s@' % key, value)

        return buf

    def isofiles(self, outdir, tmp_dir=None):
        with open(os.path.join(outdir, 'autounattend.xml'), 'wb') as f:
            f.write(self._autounattend_xml())

    def set_serial_key(self, serial_key):
        if serial_key and not valid_serial_key(serial_key):
            log.error('The provided serial key has an incorrect encoding.')
            log.info('Example encoding, AAAAA-BBBBB-CCCCC-DDDDD-EEEEE.')
            return False

        # https://technet.microsoft.com/en-us/library/jj612867.aspx
        self.serial_key = serial_key or '33PXH-7Y6KF-2VJC9-XBBR8-HVTHH'
        return True



#!/usr/bin/env python
# Copyright (C) 2014-2015 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

from __future__ import absolute_import
import ConfigParser
import json
import logging
import os

from vmcloak.constants import VMCLOAK_ROOT

log = logging.getLogger(__name__)
HWCONF_PATH = os.path.join(VMCLOAK_ROOT, 'data', 'hwconf')

class Configuration(object):
    def __init__(self):
        self.conf = {}

    def _process_value(self, value):
        if isinstance(value, str) and value.startswith('~'):
            return os.getenv('HOME') + value[1:]
        if value in ('true', 'True', 'on', 'yes', 'enable'):
            return True
        if value in ('false', 'False', 'off', 'no', 'disable'):
            return False
        return value

    def from_args(self, args):
        for key, value in args._get_kwargs():
            if key not in self.conf or value:
                self.conf[key] = self._process_value(value)

    def from_file(self, path):
        p = ConfigParser.ConfigParser()
        p.read(path)
        for key in p.options('vmcloak'):
            self.conf[key.replace('-', '_')] = \
                self._process_value(p.get('vmcloak', key))

    def from_defaults(self, defaults):
        for key, value in defaults.items():
            if key in self.conf and self.conf[key] is None:
                self.conf[key] = self._process_value(value)

    def __getattr__(self, key):
        return self.conf[key]

    def apply_types(self, types):
        for k, v in self.conf.items():
            if k not in types:
                continue

            try:
                self.conf[k] = types[k](v)
            except ValueError:
                log.critical('Flag %r should be an %r.', k, types[k].__name__)
                exit(1)

def vboxmanage_path(s):
    if os.path.isfile(s.vboxmanage):
        return s.vboxmanage

    if not s.cuckoo or not os.path.isdir(s.cuckoo):
        log.error('Please provide your Cuckoo root directory.')
        log.info('Or provide the path to the VBoxManage executable.')
        return

    conf_path = os.path.join(s.cuckoo, 'conf', 'virtualbox.conf')

    try:
        from lib.cuckoo.common.config import Config
        vboxmanage = Config(conf_path).virtualbox.path
    except:
        log.error('Unable to locate VBoxManage path, please '
                  'configure $CUCKOO/conf/virtualbox.conf properly.')
        exit(1)

    if not vboxmanage or not os.path.isfile(vboxmanage):
        log.error('The configured VBoxManage path in Cuckoo does not '
                  'exist, please configure $CUCKOO/conf/virtualbox.conf '
                  'properly.')
        exit(1)

    return vboxmanage

def load_hwconf(profile, dirpath=HWCONF_PATH):
    ret = {}

    if profile is not None:
        files = ['%s.json' % profile]
    else:
        # Load profiles that ship with VMCloak.
        files = os.listdir(dirpath)

        # Load local profiles.
        local_hwconf = os.path.join(os.getenv('HOME'),
                                    '.config', 'vmcloak', 'hwconf')

        if os.path.exists(local_hwconf):
            for fname in os.listdir(local_hwconf):
                files.append(os.path.join(local_hwconf, fname))

    for fname in files:
        if not fname.endswith('.json'):
            continue

        conf = json.load(open(os.path.join(dirpath, fname), 'rb'))
        for key, value in conf.items():
            if key not in ret:
                ret[key] = []

            if isinstance(value, list):
                ret[key].extend(value)
            else:
                ret[key].append(value)

    return ret



#!/usr/bin/env python
# Copyright (C) 2014-2015 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

from __future__ import absolute_import
import logging
import os.path
import random
import shutil
import subprocess
import tempfile

from vmcloak.conf import load_hwconf
from vmcloak.constants import VMCLOAK_ROOT
from vmcloak.misc import copytreelower, copytreeinto
from vmcloak.paths import get_path
from vmcloak.rand import random_serial, random_uuid

log = logging.getLogger(__name__)

class Machinery(object):
    FIELDS = {}
    vm_dir_required = True
    data_dir_required = True

    def __init__(self, name, vm_dir=None, data_dir=None, temp_dir=None):
        self.name = name
        self.vm_dir = vm_dir
        self.data_dir = data_dir

        self.network_idx = 0

        if self.vm_dir_required and not vm_dir:
            log.error('Please provide --vm-dir a directory for the '
                      'associated VM files.')
            log.info('Optionally this directory can be put into tmpfs for '
                     'improved speed.')
            exit(1)

        if self.data_dir_required and not data_dir:
            log.error('Please provide --data-dir a directory for the '
                      'harddisk and .iso files for the virtual machine.')
            log.info('It is recommended to store this directory on a regular '
                     'harddisk or SSD - not in tmpfs.')
            exit(1)

        if data_dir:
            self.iso_path = os.path.join(self.data_dir, '%s.iso' % self.name)
        else:
            _, self.iso_path = tempfile.mkstemp(suffix='.iso', dir=temp_dir)

    def vminfo(self, element=None):
        """Returns a dictionary with all available information for the
        Virtual Machine."""
        raise

    def create_vm(self):
        """Create a new Virtual Machine."""
        raise

    def delete_vm(self):
        """Delete an existing Virtual Machine and its associated files."""
        raise

    def ramsize(self, ramsize):
        """Modify the amount of RAM available for this Virtual Machine."""
        raise

    def os_type(self, os, sp):
        """Set the OS type to the OS and the Service Pack."""
        raise

    def create_hd(self, fsize):
        """Create a harddisk."""
        raise

    def immutable_hd(self):
        """Make a harddisk immutable or normal."""
        raise

    def remove_hd(self):
        """Remove a harddisk."""
        raise

    def cpus(self, count):
        """Set the number of CPUs to assign to this Virtual Machine."""
        raise

    def attach_iso(self, iso):
        """Attach a ISO file as DVDRom drive."""
        raise

    def detach_iso(self):
        """Detach the ISO file in the DVDRom drive."""
        raise

    def set_field(self, key, value):
        """Set a specific field of a Virtual Machine."""
        raise

    def modify_mac(self, mac=None):
        """Modify the MAC address of a Virtual Machine."""
        raise

    def network_index(self):
        """Get the index for the next network interface."""
        ret = self.network_idx
        self.network_idx += 1
        return ret

    def hostonly(self, macaddr=None, index=1):
        """Configure a hostonly adapter for the Virtual Machine."""
        raise

    def nat(self, macaddr=None, index=1):
        """Configure NAT for the Virtual Machine."""
        raise

    def hwvirt(self, enable=True):
        """Enable or disable the usage of Hardware Virtualization."""
        raise

    def start_vm(self, visible=False):
        """Start the associated Virtual Machine."""
        raise

    def snapshot(self, label):
        """Take a snapshot of the associated Virtual Machine."""
        raise

    def stopvm(self):
        """Stop the associated Virtual Machine."""
        raise

    def list_settings(self):
        """List all settings of a Virtual Machine."""
        raise

    def init_vm(self, profile):
        """Initialize fields as specified by `FIELDS`."""
        hwconf = load_hwconf(profile=profile)

        def _init_vm(path, fields):
            for key, value in fields.items():
                key = path + '/' + key
                if isinstance(value, dict):
                    _init_vm(key, value)
                else:
                    if isinstance(value, tuple):
                        k, v = value
                        if k not in hwconf or not hwconf[k]:
                            value = 'To be filled by O.E.M.'
                        else:
                            if k not in config:
                                config[k] = random.choice(hwconf[k])

                            value = config[k][v]

                            # Some values have to be generated randomly.
                            if value is not None:
                                if value.startswith('<SERIAL>'):
                                    length = int(value.split()[-1])
                                    value = random_serial(length)
                                elif value.startswith('<UUID>'):
                                    value = random_uuid()

                    if value is None:
                        value = "To be filled by O.E.M."

                    log.debug('Setting %r to %r.', key, value)
                    ret = self.set_field(key, value)
                    if ret:
                        log.debug(ret)

        config = {}
        _init_vm('', self.FIELDS)

class OperatingSystem(object):
    # Short name for this OS.
    name = None

    # Service Pack that is likely being used.
    service_pack = None

    # Default directory where the original ISO is mounted.
    mount = None

    # The Network Interface Card Type.
    nictype = None

    # Directory where to store the vmcloak bootstrap files.
    osdir = None

    # Additional arguments for genisoimage.
    genisoargs = []

    def __init__(self):
        self.data_path = os.path.join(VMCLOAK_ROOT, 'data')
        self.path = os.path.join(self.data_path, self.name)
        self.serial_key = None

        if self.name is None:
            raise Exception('Name has to be provided for OS handler')

        if self.osdir is None:
            raise Exception('OSDir has to be provided for OS handler')

    def configure(self, s):
        """Configure the setup with settings provided by the user."""
        self.s = s

    def isofiles(self, outdir, tmp_dir=None):
        """Abstract method for writing additional files to the newly created
        ISO file."""

    def set_serial_key(self, serial_key):
        """Abstract method for checking a serial key if provided and otherwise
        use a default serial key if available."""

    def buildiso(self, mount, newiso, bootstrap, tmp_dir=None):
        """Builds an ISO file containing all our modifications."""
        outdir = tempfile.mkdtemp(dir=tmp_dir)

        # Copy all files to our temporary directory as mounted iso files are
        # read-only and we need lowercase (aka case-insensitive) filepaths.
        copytreelower(mount, outdir)

        # Copy the boot image.
        shutil.copy(os.path.join(self.path, 'boot.img'), outdir)

        # Allow the OS handler to write additional files.
        self.isofiles(outdir, tmp_dir)

        os.makedirs(os.path.join(outdir, self.osdir, 'vmcloak'))

        data_bootstrap = os.path.join(self.data_path, 'bootstrap')
        for fname in os.listdir(data_bootstrap):
            shutil.copy(os.path.join(data_bootstrap, fname),
                        os.path.join(outdir, self.osdir, 'vmcloak', fname))

        copytreeinto(bootstrap, os.path.join(outdir, self.osdir))

        isocreate = get_path('genisoimage')
        if not isocreate:
            log.error('Either genisoimage or mkisofs is required!')
            shutil.rmtree(outdir)
            return False

        args = [
            isocreate, '-quiet', '-b', 'boot.img', '-o', newiso,
        ] + self.genisoargs + [outdir]

        try:
            # TODO Properly suppress the ISO-9660 warning.
            subprocess.check_call(args)
        except subprocess.CalledProcessError as e:
            log.error('Error creating ISO file: %s', e)
            shutil.rmtree(outdir)
            return False

        shutil.rmtree(outdir)
        return True



# Copyright (C) 2010-2014 Cuckoo Foundation.
# Copyright (C) 2014-2015 Jurriaan Bremer.
# This file is part of Cuckoo Sandbox - http://www.cuckoosandbox.org
# See the file 'docs/LICENSE' for copying permission.

import json
import os
import platform
import random
import socket
import string
import sys
import time
import subprocess
from ConfigParser import RawConfigParser
from SimpleXMLRPCServer import SimpleXMLRPCServer
from StringIO import StringIO
from _winreg import CreateKeyEx, DeleteValue, SetValueEx
from _winreg import HKEY_LOCAL_MACHINE, KEY_ALL_ACCESS, REG_SZ
from zipfile import ZipFile

BIND_IP = "0.0.0.0"
BIND_PORT = 8000

STATUS_INIT = 0x0001
STATUS_RUNNING = 0x0002
STATUS_COMPLETED = 0x0003
STATUS_FAILED = 0x0004
CURRENT_STATUS = STATUS_INIT

ERROR_MESSAGE = ""
ANALYZER_FOLDER = ""
RESULTS_FOLDER = ""

def delete_key(rootkey, subkey, key):
    h = CreateKeyEx(rootkey, subkey, 0, KEY_ALL_ACCESS)
    DeleteValue(h, key)
    h.Close()

def update_key(rootkey, subkey, key, value):
    h = CreateKeyEx(rootkey, subkey, 0, KEY_ALL_ACCESS)
    SetValueEx(h, key, 0, REG_SZ, value)
    h.Close()

class Agent:
    """Cuckoo agent, it runs inside guest."""

    def __init__(self):
        self.system = platform.system().lower()
        self.analyzer_path = ""
        self.analyzer_pid = 0

    def _initialize(self):
        global ERROR_MESSAGE
        global ANALYZER_FOLDER

        if not ANALYZER_FOLDER:
            random.seed(time.time())
            container = "".join(random.choice(string.ascii_lowercase) for x in range(random.randint(5, 10)))

            if self.system == "windows":
                system_drive = os.environ["SYSTEMDRIVE"] + os.sep
                ANALYZER_FOLDER = os.path.join(system_drive, container)
            elif self.system == "linux" or self.system == "darwin":
                ANALYZER_FOLDER = os.path.join(os.environ["HOME"], container)
            else:
                ERROR_MESSAGE = "Unable to identify operating system"
                return False

            try:
                os.makedirs(ANALYZER_FOLDER)
            except OSError as e:
                ERROR_MESSAGE = e
                return False

        return True

    def get_status(self):
        """Get current status.
        @return: status.
        """
        return CURRENT_STATUS

    def get_error(self):
        """Get error message.
        @return: error message.
        """
        return str(ERROR_MESSAGE)

    def add_malware(self, data, name):
        """Get analysis data.
        @param data: analysis data.
        @param name: file name.
        @return: operation status.
        """
        global ERROR_MESSAGE
        data = data.data

        if self.system == "windows":
            root = os.environ["TEMP"]
        elif self.system == "linux" or self.system == "darwin":
            root = "/tmp"
        else:
            ERROR_MESSAGE = "Unable to write malware to disk because of " \
                            "failed identification of the operating system"
            return False

        file_path = os.path.join(root, name)

        try:
            with open(file_path, "wb") as malware:
                malware.write(data)
        except IOError as e:
            ERROR_MESSAGE = "Unable to write malware to disk: {0}".format(e)
            return False

        return True

    def add_config(self, options):
        """Creates analysis.conf file from current analysis options.
        @param options: current configuration options, dict format.
        @return: operation status.
        """
        global ERROR_MESSAGE

        if type(options) != dict:
            return False

        config = RawConfigParser()
        config.add_section("analysis")

        try:
            for key, value in options.items():
                # Options can be UTF encoded.
                if isinstance(value, basestring):
                    try:
                        value = value.encode("utf-8")
                    except UnicodeEncodeError:
                        pass

                config.set("analysis", key, value)

            config_path = os.path.join(ANALYZER_FOLDER, "analysis.conf")

            with open(config_path, "wb") as config_file:
                config.write(config_file)
        except Exception as e:
            ERROR_MESSAGE = str(e)
            return False

        return True

    def add_analyzer(self, data):
        """Add analyzer.
        @param data: analyzer data.
        @return: operation status.
        """
        data = data.data

        if not self._initialize():
            return False

        try:
            zip_data = StringIO()
            zip_data.write(data)

            with ZipFile(zip_data, "r") as archive:
                archive.extractall(ANALYZER_FOLDER)
        finally:
            zip_data.close()

        self.analyzer_path = os.path.join(ANALYZER_FOLDER, "analyzer.py")

        return True

    def execute(self):
        """Execute analysis.
        @return: analyzer PID.
        """
        global ERROR_MESSAGE
        global CURRENT_STATUS

        if not self.analyzer_path or not os.path.exists(self.analyzer_path):
            return False

        # Remove this file and its associated registry key as we're about
        # to execute a sample.
        if s['vmmode'] != 'longterm':
            os.unlink(os.path.abspath(__file__))
            delete_key(HKEY_LOCAL_MACHINE,
                       'Software\\Microsoft\\Windows\\CurrentVersion\\Run',
                       'Agent')

        try:
            proc = subprocess.Popen([sys.executable, self.analyzer_path],
                                    cwd=os.path.dirname(self.analyzer_path))
            self.analyzer_pid = proc.pid
        except OSError as e:
            ERROR_MESSAGE = str(e)
            return False

        CURRENT_STATUS = STATUS_RUNNING

        return self.analyzer_pid

    def complete(self, success=True, error="", results=""):
        """Complete analysis.
        @param success: success status.
        @param error: error status.
        """
        global ERROR_MESSAGE
        global CURRENT_STATUS
        global RESULTS_FOLDER

        if success:
            CURRENT_STATUS = STATUS_COMPLETED
        else:
            if error:
                ERROR_MESSAGE = str(error)

            CURRENT_STATUS = STATUS_FAILED

        RESULTS_FOLDER = results

        return True

if __name__ == "__main__":
    # Load the configuration values.
    s = json.loads(sys.argv[1].decode('base64'))

    require_reboot = False

    # Attempt to connect to the host machine.
    if 'host_ip' in s and 'host_port' in s:
        sock = None
        while not sock:
            try:
                addr = s['host_ip'], s['host_port']
                sock = socket.create_connection(addr, 1)
            except socket.error:
                time.sleep(0.1)
                continue

        # Connect to the host machine. In case this is a bird, also
        # receive the new IP address, mask, and gateway.
        if s['vmmode'] == 'bird':
            # Retrieve the configuration for this clone.
            conf = json.loads(sock.recv(0x10000))

            # Update the VM mode to whatever vmcloak informs us to.
            s['vmmode'] = conf['vmmode']

            sock.close()

            args = [
                "netsh", "interface", "ip", "set", "address",
                "name=Local Area Connection", "static",
                conf["ip"], conf["netmask"], conf["gateway"], "1",
            ]
            subprocess.Popen(args).wait()

            if "hostname" in conf:
                # For excellent reasons (?) this only works when passed along
                # as shell command, so here we go.
                subprocess.call(
                    'wmic computersystem where name="%s" '
                    'call rename name="%s"' % (
                        os.getenv("COMPUTERNAME"), conf["hostname"]
                    ),
                    shell=True
                )

                require_reboot = True
        else:
            sock.close()

        del s['host_ip']
        del s['host_port']

    # Update the registry to reflect any changes to IP addresses, i.e., the
    # Virtual Machine connected to the host.
    value = '"%s" "%s" %s' % (sys.executable,
                              os.path.abspath(__file__),
                              json.dumps(s).encode('base64'))
    update_key(HKEY_LOCAL_MACHINE,
               'Software\\Microsoft\\Windows\\CurrentVersion\\Run',
               'Agent', value)

    # This should be further improved. Namely, per-instance changes to the
    # registry etc (although it would be easier to do that as a kernel driver).
    if require_reboot:
        subprocess.Popen(['shutdown', '-r', '-t', '0']).wait()

    try:
        if not BIND_IP:
            BIND_IP = socket.gethostbyname(socket.gethostname())

        print("[+] Starting agent on %s:%s ..." % (BIND_IP, BIND_PORT))

        # Disable DNS lookup, by Scott D.
        def FakeGetFQDN(name=""):
            return name
        socket.getfqdn = FakeGetFQDN

        server = SimpleXMLRPCServer((BIND_IP, BIND_PORT), allow_none=True)
        server.register_instance(Agent())
        server.serve_forever()
    except KeyboardInterrupt:
        server.shutdown()



# Copyright (C) 2014-2015 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.


VBOX_CONFIG = {
    'VBoxInternal/Devices/pcbios/0/Config': dict(

        # http://blog.prowling.nu/2012/08/modifying-virtualbox-settings-for.html
        DmiBIOSVendor=('bios', 'vendor'),
        DmiBIOSVersion=('bios', 'version'),
        DmiBIOSReleaseDate=('bios', 'release_date'),
        # DmiBIOSReleaseMajor=
        # DmiBIOSReleaseMinor=
        # DmiBIOSFirmwareMajor=
        # DmiBIOSFirmwareMinor=

        DmiSystemVendor=('system', 'vendor'),
        DmiSystemProduct=('system', 'product'),
        DmiSystemVersion=('system', 'version'),
        DmiSystemSerial=('system', 'serial'),
        DmiSystemSKU=('system', 'sku'),
        DmiSystemFamily=('system', 'family'),
        DmiSystemUuid=('system', 'uuid'),

        # http://blog.prowling.nu/2012/10/modifying-virtualbox-settings-for.html
        DmiBoardVendor=('board', 'vendor'),
        DmiBoardProduct=('board', 'product'),
        DmiBoardVersion=('board', 'version'),
        DmiBoardSerial=('board', 'serial'),
        DmiBoardAssetTag=('board', 'asset'),
        DmiBoardLocInChass=('board', 'location'),

        DmiChassisVendor=('chassis', 'vendor'),
        DmiChassisVersion=('chassis', 'version'),
        DmiChassisSerial=('chassis', 'serial'),
        DmiChassisAssetTag=('chassis', 'asset'),
    ),
    'VBoxInternal/Devices/piix3ide/0/Config': {
        'Port0': dict(

            # http://downloads.cuckoosandbox.org/slides/blackhat.pdf, Page 82
            # https://forums.virtualbox.org/viewtopic.php?f=1&t=48718
            # ATAPIProductId='',
            # ATAPIRevision='',
            # ATAPIVendorId='',
        ),
        'PrimaryMaster': dict(

            # http://blog.prowling.nu/2012/08/modifying-virtualbox-settings-for.html
            SerialNumber=('harddisk', 'serial'),
            FirmwareRevision=('harddisk', 'revision'),
            ModelNumber=('harddisk', 'model'),
        ),
    },
}



#!/usr/bin/env python
# Copyright (C) 2014-2015 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

from __future__ import absolute_import
import logging
import os
import shutil
import subprocess
import urlparse

from vmcloak.misc import ini_read_dict, sha1_file
from vmcloak.paths import get_path

log = logging.getLogger(__name__)

class DependencyManager(object):
    FILES = 'conf.ini', 'repo.ini', 'hashes.txt'

    def __init__(self, deps_directory=None, deps_repository=None):
        self.conf = {}
        self.repo = {}

        self.deps_directory = deps_directory
        self.deps_repository = deps_repository

        self._load_config()

    def _read_conf_file(self, path):
        ret = {}
        for line in open(path, 'rb'):
            line = line.strip()
            if not line or line.startswith('#'):
                continue

            key, value = line.split('=', 1)
            ret[key.strip()] = value.strip()
        return ret

    def _load_config(self):
        if os.path.isdir(self.deps_directory) and not self.repo:
            self.conf = \
                ini_read_dict(os.path.join(self.deps_directory, 'conf.ini'))
            self.repo = \
                ini_read_dict(os.path.join(self.deps_directory, 'repo.ini'))

            self.hashes = self._read_conf_file(
                os.path.join(self.deps_directory, 'hashes.txt'))

            files_dir = os.path.join(self.deps_directory, 'files')
            if not os.path.isdir(files_dir):
                os.mkdir(files_dir)

    def _wget(self, filename, url=None, subdir=None):
        if subdir is None:
            path = os.path.join(self.deps_directory, filename)
        else:
            path = os.path.join(self.deps_directory, subdir, filename)

        if url is None:
            url = urlparse.urljoin(self.deps_repository, filename)

        args = get_path('wget'), '-O', path, url, '--no-check-certificate'
        subprocess.check_call(args)

    def init(self, bitsize_64):
        """Initializes the dependency repository."""
        self.bitsize_64 = bitsize_64
        if not os.path.isdir(self.deps_directory):
            os.mkdir(self.deps_directory)
            try:
                log.info('Cloning vmcloak-deps.')
                for fname in self.FILES:
                    self._wget(fname)
            except subprocess.CalledProcessError as e:
                log.error('Error cloning vmcloak-deps: %s.', e)
                return False

        self._load_config()
        return True

    def update(self):
        """Updates the dependency repository."""
        if not self._check():
            return False

        try:
            log.info('Updating vmcloak-deps.')
            for fname in self.FILES:
                self._wget(fname)
        except subprocess.CalledProcessError as e:
            log.error('Error updating the vmcloak-deps repository: %s', e)
            return False

        return True

    def available(self, dependency):
        """Checks whether a dependency is available and whether its integrity
        is correct."""
        self._load_config()

        if dependency not in self.repo:
            log.info('No such dependency: %s.', dependency)
            return False

        if self.bitsize_64 and 'filename64' in self.repo[dependency]:
            fname = self.repo[dependency]['filename64']
        else:
            fname = self.repo[dependency]['filename']

        filepath = os.path.join(self.deps_directory, 'files', fname)

        if not os.path.exists(filepath):
            return False

        sha1 = sha1_file(filepath)
        if sha1 != self.hashes[fname]:
            log.warning('File %s of dependency %r downloaded with '
                        'an incorrect sha1.', fname, dependency)
            os.unlink(filepath)
            return False

        return True

    def fetch(self, dependency):
        """Fetch a single dependency."""
        if not self._check():
            return False

        self._load_config()

        if dependency not in self.repo:
            log.warning('No such dependency: %s.', dependency)
            return False

        info = self.repo[dependency]

        if self.bitsize_64 and 'filename64' in self.repo[dependency]:
            fname = self.repo[dependency]['filename64']
        else:
            fname = self.repo[dependency]['filename']

        if self.available(dependency):
            log.info('Dependency %r has already been fetched.', dependency)
            return True

        url = self.conf['vmcloak-files']['raw'] % info['filename']

        try:
            log.debug('Fetching dependency %r: %s.', dependency, fname)
            self._wget(fname, url=url, subdir='files')
        except subprocess.CalledProcessError as e:
            log.warning('Error downloading %s: %s.', info['filename'], e)
            return False

        if not self.available(dependency):
            return False

        return True

    def _check(self):
        """Checks whether the dependency repository has been initialized."""
        if not os.path.isdir(self.deps_directory):
            log.debug('Initializing the vmcloak-deps repository.')
            self.init()
            return False

        return True

class DependencyWriter(object):
    def __init__(self, dm, bootstrap_path, bitsize_64):
        self.bootstrap = bootstrap_path
        self.bitsize_64 = bitsize_64

        self.installed = []
        self.f = open(os.path.join(bootstrap_path, 'deps.bat'), 'wb')

        self.dm = dm

        # Make sure the dependency repository is available.
        self.dm.init(self.bitsize_64)

        # Copy the repository information from the dependency manager.
        self.repo = self.dm.repo

    def add(self, dependency):
        if dependency not in self.repo:
            log.error('Dependency %s not found!', dependency)
            return False

        if dependency in self.installed:
            log.debug('Dependency %s has already been handled.', dependency)
            return True

        if not self.dm.available(dependency):
            if not self.dm.fetch(dependency):
                return False

        kw = self.repo[dependency]

        fname = kw.pop('filename')
        arguments = kw.pop('arguments', '')
        depends = kw.pop('dependencies', '')
        marker = kw.pop('marker', None)
        flags = []
        cmds = []

        for flag in kw.pop('flags', '').split():
            if flag.strip():
                flags.append(flag.strip())

        idx = 0
        while 'cmd%d' % idx in kw:
            if self.bitsize_64 and 'cmd%d_64' % idx in kw:
                cmds.append(kw.pop('cmd%d_64' % idx))
            else:
                cmds.append(kw.pop('cmd%d' % idx))

            kw.pop('cmd%d' % idx, None)
            kw.pop('cmd%d_64' % idx, None)
            idx += 1

        # Not used by us.
        kw.pop('description', None)

        # If this is a 64-bit OS and there's a 64-bit installer available,
        # then use it.
        if self.bitsize_64 and kw.get('filename64'):
            fname = kw.pop('filename64')
        else:
            kw.pop('filename64', None)

        if kw:
            log.error('Found one or more remaining value(s) in the '
                      'configuration, please it fix before continuing..')
            log.info('Remaining value(s): %s', kw)
            exit(1)

        for dep in depends.split():
            if dep.strip():
                self.add(dep.strip())

        log.debug("Added dependency %r (filename %r).", dependency, fname)

        self.installed.append(dependency)

        print>>self.f, 'echo Installing..', fname
        if marker:
            print>>self.f, 'if exist "%s" (' % marker
            print>>self.f, '  echo Dependency already installed!'
            print>>self.f, ') else ('

        if 'background' in flags:
            print>>self.f, '  start C:\\vmcloak\\deps\\%s' % fname, arguments
        else:
            print>>self.f, '  C:\\vmcloak\\deps\\%s' % fname, arguments

        for cmd in cmds:
            if cmd.startswith('click'):
                print>>self.f, '  C:\\vmcloak\\%s' % cmd
            else:
                print>>self.f, '  %s' % cmd

        if marker:
            print>>self.f, ')'

        if not os.path.isdir(os.path.join(self.bootstrap, 'deps')):
            os.mkdir(os.path.join(self.bootstrap, 'deps'))

        shutil.copy(os.path.join(self.dm.deps_directory, 'files', fname),
                    os.path.join(self.bootstrap, 'deps', fname))
        return True

    def write(self):
        self.f.close()



# Copyright (C) 2014-2015 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

import os.path
import pkg_resources

def resource_path(*path):
    return pkg_resources.resource_filename('vmcloak', os.path.join(*path))

def load_resource(*path):
    return pkg_resources.resource_string('vmcloak', os.path.join(*path))

def list_resources(*path):
    return pkg_resources.resource_listdir('vmcloak', os.path.join(*path))



# Copyright (C) 2014-2015 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

from __future__ import absolute_import
import logging
import os


log = logging.getLogger(__name__)

PATHS = {
    'wget': [
        '/usr/bin/wget',
        '/usr/local/bin/wget',
        '/usr/pkg/bin/wget'
    ],
    'genisoimage': [
        '/usr/bin/genisoimage',
        '/usr/pkg/bin/genisoimage',
        '/usr/bin/mkisofs',
        '/usr/local/bin/mkisofs',
        '/usr/pkg/bin/mkisofs',
    ],
}

INSTALL = {
    'wget': {
        'Linux': 'apt-get install wget',
        'NetBSD': 'pkgin install wget',
    },
    'genisoimage': {
        'Linux': 'apt-get install genisoimage',
        'NetBSD': 'pkgin install cdrkit',
    },
}

def get_path(app):
    """Returns the path to an application."""
    for path in PATHS[app]:
        if os.path.isfile(path):
            return path

    log.error('No executable path found for %s!', app)
    log.info('In order to install it, run %r',
             INSTALL[app].get(os.uname()[0], '?'))



# Copyright (C) 2014-2015 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

from __future__ import absolute_import
import argparse
import hashlib
import json
import logging
import os
import pwd
import shutil
import socket
import stat
import subprocess
from ConfigParser import ConfigParser

from vmcloak.conf import Configuration
from vmcloak.constants import VMCLOAK_ROOT

log = logging.getLogger(__name__)

def copytreelower(srcdir, dstdir):
    """Copies the source directory as lowercase to the destination directory.

    Lowercase as in, all directory and filenames are translated to lowercase,
    thus emulating Windows case-insensitive filepaths.

    """
    # If the source directory doesn't end with a slash then we have to take
    # this into account when slicing it later on. Namely, when the source
    # directory is "/mnt/winxp" then we slice up to "/mnt/winxp/", thus we
    # require one extra character for the slash. When the source directory is
    # "/mnt/winxp/" then of course we don't an extra character for the slice.
    prefix = len(srcdir) + (not srcdir.endswith('/'))
    for dirpath, dirnames, filenames in os.walk(srcdir):
        for dirname in dirnames:
            os.mkdir(os.path.join(dstdir,
                                  dirpath[prefix:].lower(),
                                  dirname.lower()))

        for fname in filenames:
            path = os.path.join(dirpath, fname)[prefix:]

            # Copy the file.
            shutil.copyfile(os.path.join(srcdir, path),
                            os.path.join(dstdir, path.lower()))

            # Make the file writable.
            os.chmod(os.path.join(dstdir, path.lower()),
                     stat.S_IRUSR | stat.S_IWUSR)

def copytreeinto(srcdir, dstdir):
    """Copy one directory into another directory.

    Unlike shutil.copytree() this function doesn't require the destination
    directory to be non-existing.

    """
    if os.path.isfile(dstdir):
        raise Exception('Cannot create directory if there is already '
                        'a file: %s' % dstdir)

    if not os.path.isdir(dstdir):
        os.mkdir(dstdir)

    for fname in os.listdir(srcdir):
        path_in = os.path.join(srcdir, fname)
        path_out = os.path.join(dstdir, fname)

        if os.path.isfile(path_in):
            shutil.copy(path_in, path_out)
        else:
            copytreeinto(path_in, path_out)

def ini_read(path):
    ret, section = {}, None

    if os.path.exists(path):
        buf = open(path, 'rb').read()
    else:
        buf = ''

    # UTF-16 Byte Order Mark ("BOM")
    mode = 'utf16' if buf[:2] == '\xff\xfe' else 'latin1'
    buf = buf.decode(mode)

    for line in buf.split('\n'):
        line = line.strip()
        if not line or line[0] == ';':
            continue

        if line[0] == '[' and ']' in line:
            section = line[1:line.index(']')]
            ret[section] = []
            continue

        if '=' not in line:
            ret[section].append(line)
        else:
            a, b = line.split('=', 1)
            ret[section].append('%s=%s' % (a.strip(), b.strip()))
    return mode, ret

def ini_write(path, mode, data):
    lines = ['']
    for key in sorted(data.keys()):
        lines.append('[%s]' % key)
        for value in sorted(data[key]):
            lines.append(value)
        lines.append('')
    open(path, 'wb').write('\r\n'.join(lines).encode(mode))

def ini_add(data, section, value):
    if section not in data:
        data[section] = []

    if value not in data[section]:
        data[section].append(value)

def ini_delete(data, section, value):
    if section not in data:
        return

    for idx, row in enumerate(data[section]):
        if row == value:
            del data[section][idx]

def ini_merge(data, ini2, overwrite=True):
    mode, data2 = ini_read(ini2)
    for section in data2:
        for value in data2[section]:
            if section not in data:
                data[section] = [value]
                continue

            off = value.find('=')
            # If this line is not a key = value entry, then
            # we take the entire line.
            if off < 0:
                off = len(value)

            for idx, row in enumerate(data[section]):
                if len(row) > off and row[:off] == value[:off]:
                    if overwrite:
                        data[section][idx] = value
                    break
            else:
                data[section].append(value)

def ini_read_dict(path):
    c = ConfigParser()
    c.read(path)

    ret = {}
    for section in c.sections():
        ret[section] = dict(c.items(section))
    return ret

def sha1_file(path):
    """Calculate the sha1 hash of a file."""
    h = hashlib.sha1()
    f = open(path, 'rb')

    while True:
        buf = f.read(1024*1024)
        if not buf:
            break

        h.update(buf)

    return h.hexdigest()

def read_birds():
    path = os.path.join(os.getenv('HOME'), '.vmcloak', 'birds.json')
    birds = {}

    if os.path.isfile(path):
        birds = json.load(open(path, 'rb'))

    return birds

def read_bird(name):
    return read_birds().get(name)

def add_bird(name, vmtype, hdd_path):
    path = os.path.join(os.getenv('HOME'), '.vmcloak', 'birds.json')

    birds = read_birds()
    birds[name] = dict(vmtype=vmtype, hdd_path=hdd_path)

    open(path, 'wb').write(json.dumps(birds))

def shared_parameters():
    parser = argparse.ArgumentParser()
    parser.add_argument('vmname', type=str, nargs='?', help='Name of the Virtual Machine.')
    parser.add_argument('--cuckoo', type=str, help='Directory where Cuckoo is located.')
    parser.add_argument('--vm-dir', type=str, help='Base directory for the virtual machine and its associated files.')
    parser.add_argument('--data-dir', type=str, help='Base directory for the virtual machine harddisks and images.')
    parser.add_argument('--vm', type=str, help='Virtual Machine Software (VirtualBox.)')
    parser.add_argument('--ramsize', help='Available virtual memory (in MB) for this virtual machine.')
    parser.add_argument('--resolution', type=str, help='Virtual Machine resolution.')
    parser.add_argument('--hdsize', help='Maximum size (in MB) of the dynamically allocated harddisk.')
    parser.add_argument('--host-ip', type=str, help='Static IP address to bind to on the Host.')
    parser.add_argument('--host-port', type=str, help='Port to bind to on the Host.')
    parser.add_argument('--hostonly-ip', type=str, help='Static IP address to use on the Guest for the hostonly network.')
    parser.add_argument('--hostonly-mask', type=str, help='Static IP address mask to use on the Guest for the hostonly network.')
    parser.add_argument('--hostonly-gateway', type=str, help='Static IP address gateway to use on the Guest for the hostonly network.')
    parser.add_argument('--hostonly-macaddr', type=str, help='Mac address for the hostonly interface.')
    parser.add_argument('--hostonly-adapter', type=str, help='Hostonly interface to use.')
    parser.add_argument('--nat', action='store_true', help='Name of the NAT network to attach to.')
    parser.add_argument('--dns-server', type=str, help='Address of DNS server to be used.')
    parser.add_argument('--hwvirt', action='store_true', default=None, help='Explicitly enable Hardware Virtualization.')
    parser.add_argument('--no-hwvirt', action='store_false', default=None, dest='hwvirt', help='Explicitly disable Hardware Virtualization.')
    parser.add_argument('--tags', type=str, help='Cuckoo Tags for the Virtual Machine.')
    parser.add_argument('--no-register-cuckoo', action='store_false', default=None, dest='register_cuckoo', help='Explicitly disable registering the Virtual Machine with Cuckoo upon completion.')
    parser.add_argument('--vboxmanage', type=str, help='Path to VBoxManage.')
    parser.add_argument('--vm-visible', action='store_true', default=None, help='Start the Virtual Machine in GUI mode.')
    parser.add_argument('--keyboard-layout', type=str, help='Keyboard Layout within the Virtual Machine.')
    parser.add_argument('--cpu-count', help='Number of CPUs to use with this Virtual Machine.')
    parser.add_argument('--vrde', action='store_true', help='Enable VRDE support in VirtualBox.')
    parser.add_argument('--vrde-port', type=int, help='Port for the VRDE server.')
    parser.add_argument('--vrde-password', type=str, help='Password for the VRDE server.')
    parser.add_argument('--hwconfig-profile', type=str, help='Take a particular hardware profile.')
    parser.add_argument('--auxiliary', type=str, help='Path to a directory containing auxiliary files that should be shipped to the Virtual Machine.')
    parser.add_argument('--auxiliary-local', type=str, help='Overwrite the directory path to the auxiliary files in the Virtual Machine.')
    parser.add_argument('--temp-dirpath', type=str, help='Directory where to put temporary files.')
    parser.add_argument('--deps-directory', type=str, help='Dependency directory.')
    parser.add_argument('--deps-repository', type=str, help='Dependency repository.')
    parser.add_argument('--run-executable', type=str, help='Extra executable to be ran after full initialization of the Virtual Machine.')
    parser.add_argument('-s', '--settings', type=str, default=[], action='append', help='Configuration file with various settings.')
    parser.add_argument('-r', '--recommended-settings', action='store_true', help='Use the recommended settings.')
    parser.add_argument('-u', '--user', type=str, help='Drop user privileges to this user')
    parser.add_argument('-v', '--verbose', action='store_true', help='Display debug messages.')

    defaults = dict(
        vm='virtualbox',
        cuckoo='',
        ramsize=1024,
        resolution='1024x768',
        hdsize=256*1024,
        host_ip='192.168.56.1',
        host_port=0x4141,
        host_init_ip='192.168.56.2',
        host_init_mask='255.255.255.0',
        host_init_gateway='192.168.56.1',
        hostonly_ip='192.168.56.101',
        hostonly_mask='255.255.255.0',
        hostonly_gateway='192.168.56.1',
        hostonly_adapter=None,
        dns_server='8.8.8.8',
        tags='',
        vboxmanage='/usr/bin/VBoxManage',
        vm_visible=False,
        keyboard_layout='US',
        cpu_count=1,
        register_cuckoo=True,
        dependencies='',
        auxiliary_local='auxiliary',
        deps_directory='~/.vmcloak/deps',
        deps_repository='https://raw.githubusercontent.com/jbremer/vmcloak-deps/master/',
        vrde=False,
        vrde_port=3389,
        vrde_password='vmcloak',
    )

    types = dict(
        ramsize=int,
        hdsize=int,
        cpu_count=int,
        host_port=int,
    )
    return parser, defaults, types

def register_cuckoo(hostonly_ip, tags, vmname, cuckoo_dirpath, rdp_port=None):
    log.debug('Registering the Virtual Machine with Cuckoo.')
    try:
        machine_py = os.path.join(cuckoo_dirpath, 'utils', 'machine.py')
        args = [
            machine_py, '--add',
            '--ip', hostonly_ip,
            '--platform', 'windows',
            '--tags', tags,
            '--snapshot', 'vmcloak',
            vmname,
        ]

        if rdp_port:
            args += ['--rdp_port', '%s' % rdp_port]

        subprocess.check_call(args, cwd=cuckoo_dirpath)
        return True
    except OSError as e:
        log.error('Is $CUCKOO/utils/machine.py executable? -> %s', e)
        return False
    except subprocess.CalledProcessError as e:
        log.error('Error registering the VM: %s.', e)
        return False

def wait_for_host(ip):
    # Wait for the XMLRPC agent to come up with a timeout of 1 second.
    while True:
        try:
            socket.create_connection((ip, 8000), 1).close()
            break
        except socket.error:
            pass

def resolve_parameters(args, defaults, types, drop_user=False):
    s = Configuration()

    if args.recommended_settings:
        s.from_file(os.path.join(VMCLOAK_ROOT, 'data', 'recommended.ini'))

    for settings in args.settings:
        s.from_file(settings)

    s.from_args(args)
    s.from_defaults(defaults)
    s.apply_types(types)

    if s.user and not drop_user:
        try:
            user = pwd.getpwnam(s.user)
            os.setgroups((user.pw_gid,))
            os.setgid(user.pw_gid)
            os.setuid(user.pw_uid)
            os.environ['HOME'] = user.pw_dir
        except KeyError:
            raise Exception('Invalid user specified to drop '
                            'privileges to: %s' % s.user)
        except OSError as e:
            raise Exception('Failed to drop privileges: %s' % e)

        # Resolve the parameters again, but this time without applying the
        # user argument. This way paths that use os.getenv('HOME') will be
        # correct.
        return resolve_parameters(args, defaults, types, drop_user=True)

    return s



#!/usr/bin/env python
# Copyright (C) 2014-2015 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

from __future__ import absolute_import
import logging
import os
import subprocess

from vmcloak.abstract import Machinery
from vmcloak.data.config import VBOX_CONFIG
from vmcloak.exceptions import CommandError
from vmcloak.rand import random_mac

log = logging.getLogger(__name__)

class VirtualBox(Machinery):
    FIELDS = VBOX_CONFIG

    def __init__(self, *args, **kwargs):
        self.vboxmanage = kwargs.pop('vboxmanage')
        Machinery.__init__(self, *args, **kwargs)

        self.hdd_path = os.path.join(self.data_dir, '%s.vdi' % self.name)

    def _call(self, *args, **kwargs):
        cmd = [self.vboxmanage] + list(args)

        for k, v in kwargs.items():
            if v is None or v is True:
                cmd += ['--' + k]
            else:
                cmd += ['--' + k.rstrip('_'), str(v)]

        try:
            ret = subprocess.check_output(cmd)
        except Exception as e:
            log.error('[-] Error running command: %s', e)
            raise CommandError

        return ret.strip()

    def vminfo(self, element=None):
        ret = {}
        lines = self._call('showvminfo', self.name, machinereadable=True)
        for line in lines.split('\n'):
            key, value = line.split('=', 1)

            if value.startswith('"') and value.endswith('"'):
                value = value[1:-1]
            elif value.isdigit():
                value = int(value)

            if key.startswith('"') and key.endswith('"'):
                key = key[1:-1]

            ret[key] = value
        return ret if element is None else ret.get(element)

    def api_status(self):
        if not os.path.isfile(self.vboxmanage):
            log.error('VBoxManage not found!')
            return False

        return True

    def create_vm(self):
        return self._call('createvm', name=self.name,
                          basefolder=self.vm_dir, register=True)

    def delete_vm(self):
        self._call('unregistervm', self.name, delete=True)

    def ramsize(self, ramsize):
        return self._call('modifyvm', self.name, memory=ramsize)

    def os_type(self, os, sp):
        operating_systems = {
            'winxp': 'WindowsXP',
            'win7': 'Windows7',
        }
        return self._call('modifyvm', self.name, ostype=operating_systems[os])

    def create_hd(self, fsize):
        self._call('createhd', filename=self.hdd_path, size=fsize)
        self._call('storagectl', self.name, name='IDE', add='ide')
        self._call('storageattach', self.name, storagectl='IDE',
                   type_='hdd', device=0, port=0, medium=self.hdd_path)

    def attach_hd(self, path):
        self._call('storagectl', self.name, name='IDE', add='ide')
        self._call('storageattach', self.name, storagectl='IDE',
                   type_='hdd', device=0, port=0, medium=path)

    def immutable_hd(self):
        # We first make the HDD "more" compact - this should be basically
        # defragmenting it. Only after doing so we make it multiattach. (Even
        # though it may not be technically required, the --compact switch and
        # immutable type gave problems earlier, so we're just being careful).
        self._call('modifyhd', self.hdd_path, compact=True)
        self._call('modifyhd', self.hdd_path, type_='multiattach')

    def clone_hd(self, hdd_inpath, hdd_outpath):
        self._call('clonehd', hdd_inpath, hdd_outpath)

    def remove_hd(self):
        self._call('storagectl', self.name, portcount=0,
                   name='IDE', remove=True)

    def cpus(self, count):
        self._call('modifyvm', self.name, cpus=count, ioapic='on')

    def attach_iso(self, iso):
        self._call('storageattach', self.name, storagectl='IDE',
                   type_='dvddrive', port=1, device=0, medium=iso)

    def detach_iso(self):
        self._call('storageattach', self.name, storagectl='IDE',
                   type_='dvddrive', port=1, device=0, medium='emptydrive')

    def set_field(self, key, value):
        return self._call('setextradata', self.name, key, value)

    def modify_mac(self, macaddr=None, index=1):
        if macaddr is None:
            macaddr = random_mac()

        # VBoxManage prefers MAC addresses without colons.
        vbox_mac = macaddr.replace(':', '')

        mac = {'macaddress%d' % index: vbox_mac}
        self._call('modifyvm', self.name, **mac)
        return macaddr

    def hostonly(self, nictype, macaddr=None, adapter=None):
        index = self.network_index() + 1
        if not adapter:
            if os.name == 'posix':
                adapter = 'vboxnet0'
            else:
                adapter = 'VirtualBox Host-Only Ethernet Adapter'

        # Ensure our hostonly interface is actually up and running.
        if adapter not in self._call('list', 'hostonlyifs'):
            log.error('Have you configured %s?', adapter)
            log.info('Please refer to the documentation to configure it.')
            return False

        nic = {
            'nic%d' % index: 'hostonly',
            'nictype%d' % index: nictype,
            'nicpromisc%d' % index: 'allow-all',
            'hostonlyadapter%d' % index: adapter,
        }
        self._call('modifyvm', self.name, **nic)
        return self.modify_mac(macaddr, index)

    def nat(self, nictype, macaddr=None):
        index = self.network_index() + 1

        nic = {
            'nic%d' % index: 'nat',
            'nictype%d' % index: nictype,
            'nicpromisc%d' % index: 'allow-all',
        }
        self._call('modifyvm', self.name, **nic)
        return self.modify_mac(macaddr, index)

    def hwvirt(self, enable=True):
        """Enable or disable the usage of Hardware Virtualization."""
        self._call('modifyvm', self.name, hwvirtex='on' if enable else 'off')

    def start_vm(self, visible=False):
        return self._call('startvm', self.name,
                          type_='gui' if visible else 'headless')

    def snapshot(self, label, description=''):
        return self._call('snapshot', self.name, 'take', label,
                          description=description, live=True)

    def stopvm(self):
        return self._call('controlvm', self.name, 'poweroff')

    def list_settings(self):
        return self._call('getextradata', self.name, 'enumerate')

    def mouse(self, type):
        return self._call('modifyvm', self.name, mouse=type)

    def vrde(self, port, password):
        return self._call('modifyvm', self.name, vrde='on', vrdeport=port,
                          vrdeproperty='VNCPassword=%s' % password)

def initialize_vm(m, s, h, clone=False):
    log.debug('Creating VM %r.', s.vmname)
    log.debug(m.create_vm())

    m.ramsize(s.ramsize)
    m.os_type(os=h.name, sp=h.service_pack)

    if not clone:
        log.debug('Creating Harddisk.')
        m.create_hd(s.hdsize)

        log.debug('Temporarily attaching DVD-Rom unit for the ISO installer.')
        m.attach_iso(m.iso_path)

    log.debug('Randomizing Hardware.')
    m.init_vm(profile=s.hwconfig_profile)

    log.debug('Setting CPU count to %d', s.cpu_count)
    m.cpus(s.cpu_count)

    log.debug('Checking VirtualBox hostonly network.')
    if not m.hostonly(nictype=h.nictype, macaddr=s.hostonly_macaddr,
                      adapter=s.hostonly_adapter):
        exit(1)

    if s.nat:
        m.nat(nictype=h.nictype)

    if s.hwvirt is not None:
        if s.hwvirt:
            log.debug('Enabling Hardware Virtualization.')
        else:
            log.debug('Disabling Hardware Virtualization.')
        m.hwvirt(s.hwvirt)

    if s.vrde:
        # Sets the "Absolute Pointing Device" for better vrde support.
        m.mouse('usbtablet')
        m.vrde(s.vrde_port, s.vrde_password)



# Copyright (C) 2014-2015 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

from __future__ import absolute_import
import os.path


VMCLOAK_ROOT = os.path.abspath(os.path.dirname(__file__))



#!/usr/bin/env python
# Copyright (C) 2014-2015 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

from setuptools import setup


setup(
    name='VMCloak',
    version='0.2.11',
    author='Jurriaan Bremer',
    author_email='jurriaanbremer@gmail.com',
    packages=[
        'vmcloak',
        'vmcloak.data',
        'vmcloak.data.bootstrap',
        'vmcloak.data.hwconf',
    ],
    scripts=[
        'bin/vmcloak',
        'bin/vmcloak-bird',
        'bin/vmcloak-clone',
        'bin/vmcloak-deps',
        'bin/vmcloak-gethwconf',
        'bin/vmcloak-iptables',
        'bin/vmcloak-killvbox',
        'bin/vmcloak-removevms',
        'bin/vmcloak-vboxnet0',
    ],
    url='http://vmcloak.org/',
    license='GPLv3',
    description='Automated Virtual Machine Generation and Cloaking '
                'for Cuckoo Sandbox.',
    include_package_data=True,
    package_data={
        'vmcloak.data': ['*.*'],
        'vmcloak.data.bootstrap': ['*.*'],
        'vmcloak.data.winxp': ['*.*'],
        'vmcloak.data.win7': ['*.*'],
    },
    install_requires=[
        # Libraries to build the documentation.
        'sphinx',

        # Required on BSD systems for unknown reasons.
        'jinja2',
    ],
)



# Copyright (C) 2014-2015 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

import json
import logging
import os
import random
import shutil
import stat
import string
import subprocess
import sys
import tempfile

from ctypes import c_char, c_ushort, c_uint, c_char_p, c_wchar_p
from ctypes import windll, Structure, POINTER, sizeof, byref, pointer
from ctypes.wintypes import HANDLE, DWORD, LPCWSTR, ULONG, LONG
from _winreg import CreateKeyEx, SetValueEx
from _winreg import HKEY_CURRENT_USER, HKEY_LOCAL_MACHINE, KEY_ALL_ACCESS
from _winreg import KEY_SET_VALUE, REG_DWORD, REG_SZ, REG_MULTI_SZ

import settings as s

# http://blogs.technet.com/b/heyscriptingguy/archive/2010/07/07/hey-scripting-guy-how-can-i-change-my-desktop-monitor-resolution-via-windows-powershell.aspx
# http://msdn.microsoft.com/en-us/library/windows/desktop/dd183565(v=vs.85).aspx
class _DevMode(Structure):
    _fields_ = [
        ('dmDeviceName', c_char * 32),
        ('unused1', c_ushort * 2),
        ('dmSize', c_ushort),
        ('unused2', c_ushort),
        ('unused3', c_uint * 8),
        ('dmFormName', c_char * 32),
        ('dmLogPixels', c_ushort),
        ('dmBitsPerPel', c_ushort),
        ('dmPelsWidth', c_uint),
        ('dmPelsHeight', c_uint),
        ('unused2', c_uint * 10),
    ]

class UNICODE_STRING(Structure):
    _fields_ = [
        ('Length', c_ushort),
        ('MaximumLength', c_ushort),
        ('Buffer', c_wchar_p),
    ]

EnumDisplaySettings = windll.user32.EnumDisplaySettingsA
EnumDisplaySettings.argtypes = c_char_p, c_uint, POINTER(_DevMode)

ChangeDisplaySettings = windll.user32.ChangeDisplaySettingsA
ChangeDisplaySettings.argtypes = POINTER(_DevMode), c_uint

NtRenameKey = windll.ntdll.NtRenameKey
NtRenameKey.argtypes = HANDLE, POINTER(UNICODE_STRING)

RegOpenKeyExW = windll.advapi32.RegOpenKeyExW
RegOpenKeyExW.argtypes = HANDLE, LPCWSTR, DWORD, ULONG, POINTER(HANDLE)
RegOpenKeyExW.restype = LONG

RegCloseKey = windll.advapi32.RegCloseKey
RegCloseKey.argtypes = HANDLE,

ENUM_CURRENT_SETTINGS = -1
CDS_UPDATEREGISTRY = 1
DISP_CHANGE_SUCCESSFUL = 0

HARDDISK_NAMES = [
    'HD', 'HARDDISK', 'IBM', 'WD', 'Western Digital', 'Seagate',
    'HGST', 'Samsung', 'Harddisk', 'Drive', 'TB', 'GB', 'Desk',
    'IDE', 'SATA', 'USB', 'Desktop',
]

CD_NAMES = [
    'CD', 'CDROM', 'IBM', 'WD', 'Western Digital', 'Seagate', 'HGST',
    'Samsung', 'CD-ROM', 'Drive', 'TB', 'GB', 'Desk', 'IDE', 'SATA',
    'USB', 'BlueRay', 'Blue-Ray', 'DVD', 'DVD-ROM', 'DVD ROM', 'RW',
]

BIOS_NAMES = [
    'BIOS', '3.6', '5.4', 'System', 'Version', 'Award', 'AMI', 'EFI',
    'UEFI', 'Insyde', 'SeaBIOS',
]

VGA_BIOS_NAMES = [
    'BIOS', '3.6', '5.24', 'VGA', 'Version', 'Gigabyte', 'Dell',
    'Sapphire', 'Alienware', 'Gainward', 'Asus',
]

def random_string(length=6):
    """Create silly char combinations."""
    return ''.join(random.sample(string.letters, 6))

def generate_hd():
    """Generates a random harddisk name."""
    return ' '.join(random.sample(HARDDISK_NAMES, 3))

def generate_cd():
    """Generates a random CD name."""
    return ' '.join(random.sample(CD_NAMES, 3))

def generate_bios():
    """Generates a random BIOS name."""
    return ' '.join(random.sample(BIOS_NAMES, 3))

def generate_vga_bios():
    """Generates a random VGA BIOS name."""
    return ' '.join(random.sample(VGA_BIOS_NAMES, 3))

REGISTRY = [
    # Disable "Windows XP Tour" prompt.
    # http://www.techrepublic.com/article/tech-tip-disable-the-windows-xp-tour-prompt/
    (HKEY_CURRENT_USER, 'Software\\Microsoft\\Windows\\CurrentVersion\\Applets\\Tour', 'RunCount', REG_DWORD, 0),

    # Disable the Windows Indexation service.
    # http://www.wikihow.com/Turn-Off-Windows-XP%E2%80%99s-Indexing-Service
    (HKEY_LOCAL_MACHINE, 'System\\CurrentControlSet\\Services\\CiSvc', 'Start', REG_DWORD, 4),

    # Cloak HD device identifier.
    (HKEY_LOCAL_MACHINE, 'HARDWARE\\DEVICEMAP\\Scsi\\Scsi Port 0\\Scsi Bus 0\\Target Id 0\\Logical Unit Id 0', 'Identifier', REG_SZ, generate_hd()),

    # Cloak CDROM.
    (HKEY_LOCAL_MACHINE, 'HARDWARE\\DEVICEMAP\\Scsi\\Scsi Port 1\\Scsi Bus 0\\Target Id 0\\Logical Unit Id 0', 'Identifier', REG_SZ, generate_cd()),

    # Cloak SystemBios Version.
    (HKEY_LOCAL_MACHINE, 'HARDWARE\\Description\\System', 'SystemBiosVersion', REG_MULTI_SZ, [generate_bios()]),

    # Cloak SystemBios Version.
    (HKEY_LOCAL_MACHINE, 'HARDWARE\\Description\\System', 'VideoBiosVersion', REG_MULTI_SZ, [generate_vga_bios(), generate_vga_bios()]),
]

class SetupWindows(object):
    def __init__(self, keep_evidence=False):
        self.log = logging.getLogger('Setup Windows')
        self.log.setLevel(logging.DEBUG)
        self.log_fh = logging.FileHandler('c:\\vmcloak\\windows_setup.log')
        self.log_fh.setLevel(logging.DEBUG)
        formatter = logging.Formatter('%(asctime)s - %(name)s - '
                                      '%(levelname)s - %(message)s')
        self.log_fh.setFormatter(formatter)
        self.log.addHandler(self.log_fh)
        self.keep_evidence = keep_evidence

    def set_resolution(self, width, height):
        """Set the screen resolution."""
        dm = _DevMode()
        dm.dmSize = sizeof(dm)
        if not EnumDisplaySettings(None, ENUM_CURRENT_SETTINGS, dm):
            return False

        dm.dmPelsWidth = width
        dm.dmPelsHeight = height

        ret = ChangeDisplaySettings(dm, CDS_UPDATEREGISTRY)
        return ret == DISP_CHANGE_SUCCESSFUL

    def set_regkey(self, key, subkey, name, typ, value):
        """Set a specified registry key."""
        parts = subkey.split('\\')
        for off in xrange(1, len(parts)):
            CreateKeyEx(key, '\\'.join(parts[:off]), 0, KEY_SET_VALUE).Close()

        with CreateKeyEx(key, subkey, 0, KEY_SET_VALUE) as handle:
            SetValueEx(handle, name, 0, typ, value)
            self.log.info('Set value to %r %r', key, subkey)

    def rename_regkey(self, skey, ssubkey, dsubkey):
        res_handle = HANDLE()
        options = DWORD(0)
        res = RegOpenKeyExW(skey, ssubkey, options,
                            KEY_ALL_ACCESS, byref(res_handle))
        if not res:
            bsize = c_ushort(len(dsubkey) * 2)
            us = UNICODE_STRING()
            us.Buffer = c_wchar_p(dsubkey)
            us.Length = bsize
            us.MaximumLength = bsize

            res = NtRenameKey(res_handle, pointer(us))
            if res:
                self.log.error('Could not rename %r', ssubkey)
            else:
                self.log.info('Renamed %r to %r', ssubkey, dsubkey)

        if res_handle:
            RegCloseKey(res_handle)

    def run(self):
        """Modify the system settings."""
        self.log.info('Starting system modifications')

        # Read the agent.py file so we can drop it again later on.
        agent = open('C:\\vmcloak\\agent.py', 'rb').read()

        # Set the desired resolution.
        width, height = [int(x) for x in s.resolution.split('x')]
        self.set_resolution(width, height)

        # Set registry keys.
        for key, subkey, name, typ, value in REGISTRY:
            self.set_regkey(key, subkey, name, typ, value)

        settings = dict(
            vmmode=s.vmmode,
            host_ip=s.host_ip,
            host_port=s.host_port,
        )

        fd, agent_path = \
            tempfile.mkstemp(dir=os.getenv('APPDATA'), suffix='.py')
        os.write(fd, agent)
        os.close(fd)

        # Install agent.py to be ran on the next startup.
        value = '"%s" "%s" %s' % (sys.executable, agent_path,
                                  json.dumps(settings).encode('base64'))
        self.set_regkey(HKEY_LOCAL_MACHINE,
                        'Software\\Microsoft\\Windows\\CurrentVersion\\Run',
                        'Agent', REG_SZ, value)

        # Rename registry keys.
        self.rename_regkey(HKEY_LOCAL_MACHINE,
                           'HARDWARE\\ACPI\\DSDT\\VBOX__', random_string())

        self.rename_regkey(HKEY_LOCAL_MACHINE,
                           'HARDWARE\\ACPI\\FADT\\VBOX__', random_string())

        self.rename_regkey(HKEY_LOCAL_MACHINE,
                           'HARDWARE\\ACPI\\RSDT\\VBOX__', random_string())

        # Ensure that there are no read-only flags as this would raise
        # exceptions when removing the files.
        for dirpath, _, filenames in os.walk('C:\\vmcloak'):
            for filename in filenames:
                os.chmod(os.path.join(dirpath, filename), stat.S_IWRITE)

        # Remove all vmcloak files that are directly related. This does not
        # include the auxiliary directory or any of its contents.
        if not self.keep_evidence:
            self.log_fh.close()
            shutil.rmtree('C:\\vmcloak')
        else:
            self.log.info('Keeping evidence')

        # Wait for Windows to somewhat boot the system up.
        # TODO Instead fork into another process, kill this process, and
        # shutdown from there. Now it's just hanging around.

        if s.vmmode == 'bird':
            # In bird mode we shutdown the Virtual Machine.
            subprocess.Popen(['shutdown', '-s', '-t', '0']).wait()
        else:
            # In normal mode we reboot the Virtual Machine.
            subprocess.Popen(['shutdown', '-r', '-t', '0']).wait()


if __name__ == '__main__':
    sw = SetupWindows(keep_evidence=False)
    sw.run()



# Copyright (C) 2014-2015 Jurriaan Bremer.
# This file is part of VMCloak - http://www.vmcloak.org/.
# See the file 'docs/LICENSE.txt' for copying permission.

import logging
import os
import tempfile

from vmcloak.abstract import OperatingSystem
from vmcloak.misc import ini_merge, ini_read, ini_write
from vmcloak.rand import random_string
from vmcloak.verify import valid_serial_key

log = logging.getLogger(__name__)

class WindowsXP(OperatingSystem):
    name = 'winxp'
    service_pack = 3
    mount = '/mnt/winxp'
    nictype = 'Am79C973'
    osdir = os.path.join('$oem$', '$1')
    genisoargs = [
        '-no-emul-boot', '-boot-load-seg', '1984', '-boot-load-size', '4',
        '-iso-level', '2', '-J', '-l', '-D', '-N', '-joliet-long',
        '-relaxed-filenames',
    ]

    def _winnt_sif(self):
        s = self.s
        values = {
            'PRODUCTKEY': self.serial_key,
            'COMPUTERNAME': random_string(8, 16),
            'FULLNAME': '%s %s' % (random_string(4, 8), random_string(4, 10)),
            'ORGANIZATION': '',
            'WORKGROUP': random_string(4, 8),
            'KBLAYOUT': s.keyboard_layout,
        }

        buf = open(os.path.join(self.path, 'winnt.sif'), 'rb').read()
        for key, value in values.items():
            buf = buf.replace('@%s@' % key, value)

        fd, winntsif = tempfile.mkstemp(suffix='.sif', dir=s.temp_dirpath)
        os.write(fd, buf)
        os.close(fd)

        return winntsif

    def isofiles(self, outdir, tmp_dir=None):
        dst_winnt = os.path.join(outdir, 'i386', 'winnt.sif')

        winnt_sif = self._winnt_sif()

        # Merge the existing winnt.sif with our changes.
        mode, winnt = ini_read(dst_winnt)
        ini_merge(winnt, winnt_sif, overwrite=True)

        os.remove(winnt_sif)

        # There are a couple of optional values that should be set if they have
        # not already been set.
        winnt_opt_sif = os.path.join(self.path, 'winnt-opt.sif')
        ini_merge(winnt, winnt_opt_sif, overwrite=False)

        ini_write(dst_winnt, mode, winnt)

    def set_serial_key(self, serial_key):
        if not serial_key:
            log.error('No serial key has been provided - please do so!')
            return False

        if not valid_serial_key(serial_key):
            log.error('The provided serial key has an incorrect encoding.')
            log.info('Example encoding, AAAAA-BBBBB-CCCCC-DDDDD-EEEEE.')
            return False

        self.serial_key = serial_key
        return True
